package com.sharppaint;

import java.awt.*;
import java.awt.event.*;
import java.awt.print.*;
import java.beans.*;
import java.text.NumberFormat;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.NumberFormatter;

import com.sharppaint.drawtools.DrawToolConst;
import com.sharppaint.drawtools.Element;
import com.sharppaint.drawtools.ImageTool;
import com.sharppaint.file.FileTool;
import com.sharppaint.layermanager.LayerManagementDialog;
import com.sharppaint.printpreview.PreviewDialog;
import com.sharppaint.toolbar.*;

/*
 * GNU General Public License, version 2
 * This is a vector drawing application written in Java.
 * It supports SVG, SD(SharpDraw), raster export(jpg, png, gif).
 * Copyright (C) 2007  Hee-byung Lee

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * e-mail : sky1lhb@naver.com
 * site1 : http://www.sharppaint.com/
 * site2 : http://sourceforge.net/projects/sharpdraw/
 */


/**
 *  
 *  MainWindow extends JFrame
 *  
 *  including toolbar, MenuBar, VectorProcessDialog
 */

public class MainWindow extends JFrame {
	/**
	 * 
	 */
	private static final long serialVersionUID = 8813240295797581077L;
	
	private static MainWindow mainWindow;
	
	//ֿ  
	//dialog
	private NewFileDialog newFile;
	private DrawingPanel panel; // canvas
	private WestToolBox westToolBox; // frame  toolbar class
	public UpperToolBox upperToolBox;
	public LayerManagementDialog vectorLayerDialog;
	public static JToolBar toolBar;

	//drawPanel & InternalFrame
	private ArrayList<DrawingPanel> arrDrawPanel;
	private ArrayList<JInternalFrame> arrInFrame;
	
	private JDesktopPane desktop;
	private JPanel westPanel, eastPanel, southPanel;
	private JLabel statusBar;
	private JMenuBar mb;
	private JInternalFrame inFrame;

	public Dimension screenSize;
	private String filePath;
	private FileTool myFile;
	private JScrollPane scroller;
	private Dimension panelSizeDimension;
	private int inFrameWidth;
	private int inFrameHeight;
	private int inFrameCount;
	private int inFrameLayout;
	private int inFrameNum;
	private String mode;
	private ImageIcon frameIcon; 
	private PrinterJob job;
	private PageFormat pf;
	
	//menuBar
	private JCheckBoxMenuItem anti;
	private JCheckBoxMenuItem render;
	private JCheckBoxMenuItem dither;
	private JCheckBoxMenuItem textAnti;
	private JCheckBoxMenuItem alphaInter;
	private JCheckBoxMenuItem colorRender;
	private JCheckBoxMenuItem interpolation;
	private JCheckBoxMenuItem sView;
	
	// toolbar
	private static JButton clear, open, save, print, undo, redo;
	private static JSlider resizeS;
	private static JFormattedTextField resizeT;
	private NumberFormat numberFormat;
    private NumberFormatter formatter;
	private int resizePercent;
	private ImageIcon clearIcon, openIcon, saveIcon;
	private ImageIcon printIcon;
	private ImageIcon undoIcon, redoIcon;
	
	//variable
	public static final int INSET=1;
	private MainWindow() {
		super("SharpDraw v0.9 Beta");		
		myFile = FileTool.getInstance();	
		arrDrawPanel = new ArrayList<DrawingPanel>();
		arrInFrame = new ArrayList<JInternalFrame>();
		frameIcon = new ImageIcon(
				getClass().getResource(Constants.ICON_PATH + "main.gif"));
		setIconImage(frameIcon.getImage());
		inFrameLayout = 0;
		inFrameCount = 0;
		inFrameNum = 1;
        JDialog.setDefaultLookAndFeelDecorated(true);
		create_GUI();
	}
	
	public static MainWindow getInstance() {
		if(mainWindow == null) {
			mainWindow = new MainWindow();
		}
		return mainWindow;
	}
	
	public DrawingPanel getPanel() {
		return panel;
	}
	
	public void create_GUI() { // frame,  ޴ Gui create
		screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		int inset = 10;
		setBounds(
				inset, inset, screenSize.width - inset * 2, screenSize.height
				- inset * 5);
		// JDialog.setDefaultLookAndFeelDecorated(true);
		setBackground(Color.gray);
		desktop = new JDesktopPane();
		desktop.setBackground(Color.GRAY);
		//  toolBar
		makeToolBar();
		westToolBox = WestToolBox.getInstance(); //  toolBar
		vectorLayerDialog = LayerManagementDialog.getInstance(this);
		vectorLayerDialog.setLocation(screenSize.width - LayerManagementDialog.DIALOG_WIDTH - 40, 300);
		westPanel = new JPanel();
		eastPanel = new JPanel();
		southPanel = new JPanel();
		westPanel.setBackground(this.getBackground());
		westPanel.add(westToolBox);
		statusBar = new JLabel();
		southPanel.setBorder(BorderFactory.createEtchedBorder());
		southPanel.setLayout(null);
		southPanel.setPreferredSize(new Dimension(
				screenSize.width - inset * 2, 18));
		statusBar.setText(Status.RECTANGLE);
		statusBar.setBounds(170, 0, 800, 18);
		//statusBar.setHorizontalTextPosition(SwingConstants.RIGHT);
		southPanel.add(statusBar);
		createMenuBar();
		setJMenuBar(mb);
		vectorLayerDialog.setVisible(true); 
		getContentPane().add(BorderLayout.NORTH, toolBar);
		getContentPane().add(BorderLayout.CENTER, desktop);
		getContentPane().add(BorderLayout.WEST, westPanel);
		getContentPane().add(BorderLayout.EAST, eastPanel);
		getContentPane().add(BorderLayout.SOUTH, southPanel);

		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		addWindowListener(new ExitHandler());
		setVisible(true);
	}	

	private void createMenuBar() {
		mb = new JMenuBar();
		mb.setBorder(BorderFactory.createEtchedBorder());
		mb.setSize(500, 10);
		//super.setFont()
		//super.setFont(new Font("Ÿ", Font.BOLD, 15));
		//mb.setFont(new Font("Ÿ", Font.BOLD, 11))
		JMenu file = new JMenu("File");
		file.setMnemonic(KeyEvent.VK_F);
		JMenuItem newItem = new JMenuItem("New(N)");
		newItem.setMnemonic(KeyEvent.VK_N); // Ű 
		newItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N,
				ActionEvent.CTRL_MASK));
		JMenuItem openItem = new JMenuItem("Open(O)");
		openItem.setMnemonic(KeyEvent.VK_O); // Ű 
		openItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,
				ActionEvent.CTRL_MASK));
		JMenuItem saveItem = new JMenuItem("Save(S)");
		saveItem.setMnemonic(KeyEvent.VK_S); // Ű 
		saveItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,
				ActionEvent.CTRL_MASK));
		JMenuItem saveAsItem = new JMenuItem("Save As(A)");
		saveAsItem.setMnemonic(KeyEvent.VK_A); // Ű 
		JMenuItem closeItem = new JMenuItem("Exit(X)");
		closeItem.setMnemonic(KeyEvent.VK_X); // Ű 
		closeItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F4,
				ActionEvent.ALT_MASK));
		JMenuItem previewItem = new JMenuItem("Preview(V)");
		previewItem.setMnemonic(KeyEvent.VK_G); // Ű 
		previewItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V,
				ActionEvent.CTRL_MASK+ActionEvent.ALT_MASK));
		JMenuItem printItem = new JMenuItem("Print(P)");
		printItem.setMnemonic(KeyEvent.VK_P); // Ű 
		printItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P,
				ActionEvent.CTRL_MASK));
		file.add(newItem);
		file.add(openItem);
		file.addSeparator();
		file.add(saveItem);
		file.add(saveAsItem);
		file.addSeparator();
		file.add(previewItem);
		file.add(printItem);
		file.addSeparator();
		file.add(closeItem);
		mb.add(file);
		newItem.addActionListener(new NewHandler());
		openItem.addActionListener(new OpenHandler());
		saveItem.addActionListener(new SaveHandler());
		saveAsItem.addActionListener(new SaveAsHandler());
		previewItem.addActionListener(new PreviewHandler());
		printItem.addActionListener(new PrintHandler());
		closeItem.addActionListener(new ExitHandler());

		JMenu edit = new JMenu("Edit");
		edit.setMnemonic(KeyEvent.VK_E);
		JMenuItem undoItem = new JMenuItem("Undo(U)");
		undoItem.setMnemonic(KeyEvent.VK_U); // Ű 
		undoItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_U,
				ActionEvent.CTRL_MASK));
		JMenuItem redoItem = new JMenuItem("Redo(R)");
		redoItem.setMnemonic(KeyEvent.VK_R); // Ű 
		redoItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R,
				ActionEvent.CTRL_MASK));
		edit.add(undoItem);
		edit.add(redoItem);
		mb.add(edit);
		undoItem.addActionListener(new UndoHandler());
		redoItem.addActionListener(new RedoHandler());		
		
		JMenu quality = new JMenu("Quality");
		quality.setMnemonic(KeyEvent.VK_Q);

		anti = new JCheckBoxMenuItem("AntiAliasing");
		render = new JCheckBoxMenuItem("RenderQuality");
		dither = new JCheckBoxMenuItem("Dithering");
		textAnti = new JCheckBoxMenuItem("TextAntiAliasing");
		alphaInter = new JCheckBoxMenuItem("AlphaInterpolation");
		colorRender = new JCheckBoxMenuItem("ColorRendering");
		interpolation = new JCheckBoxMenuItem("Interpolation");
		sView = new JCheckBoxMenuItem("RealView(Layer)");
		anti.setSelected(true);
		render.setSelected(false);
		dither.setSelected(false);
		textAnti.setSelected(false);
		alphaInter.setSelected(false);
		colorRender.setSelected(false);
		interpolation.setSelected(false);
		sView.setSelected(false);
		anti.addActionListener(new RenderHandler());
		render.addActionListener(new RenderHandler());
		dither.addActionListener(new RenderHandler());
		textAnti.addActionListener(new RenderHandler());
		alphaInter.addActionListener(new RenderHandler());
		colorRender.addActionListener(new RenderHandler());
		interpolation.addActionListener(new RenderHandler());
		sView.addActionListener(new RealViewHandler());
		quality.add(anti);
		quality.add(render);
		quality.add(dither);
		quality.add(textAnti);
		quality.add(alphaInter);
		quality.add(colorRender);
		quality.add(interpolation);
		quality.addSeparator();
		quality.add(sView);
		mb.add(quality);
		
		JMenu window = new JMenu("Window");
		window.setMnemonic(KeyEvent.VK_W);
		JCheckBoxMenuItem option = new JCheckBoxMenuItem("Options");
		JCheckBoxMenuItem tools = new JCheckBoxMenuItem("Tools");
		JCheckBoxMenuItem layer = new JCheckBoxMenuItem("Layers");
		option.setSelected(true);
		tools.setSelected(true);
		layer.setSelected(true);
		option.addActionListener(new ViewHandler());
		tools.addActionListener(new ViewHandler());
		layer.addActionListener(new ViewHandler());
		window.add(option);
		window.add(tools);
		window.add(layer);
		mb.add(window);	
		/*
		JMenu screen = new JMenu("Ų(S)");
		screen.setMnemonic(KeyEvent.VK_S);
		item1 = new JMenuItem("Moderate(M)");
		item2 = new JMenuItem("Sahara(H)");
		item3 = new JMenuItem("Business(B)");
		item4 = new JMenuItem("Raven(R)");
		item5 = new JMenuItem("Mango(N)");
		item6 = new JMenuItem("Substance(S)");
		javaItem = new JMenuItem("Java(J)");
		
		item1.setMnemonic(KeyEvent.VK_M); 
		item2.setMnemonic(KeyEvent.VK_H); 
		item3.setMnemonic(KeyEvent.VK_B); 
		item4.setMnemonic(KeyEvent.VK_R); 
		item5.setMnemonic(KeyEvent.VK_N); 
		item6.setMnemonic(KeyEvent.VK_S);
		javaItem.setMnemonic(KeyEvent.VK_J); 
		screen.add(javaItem);
		screen.add(item6);
		screen.add(item1);
		screen.add(item2);
		screen.add(item3);
		screen.add(item4);
		screen.add(item5);
		item1.addActionListener(new LookHandler());
		item2.addActionListener(new LookHandler());
		item3.addActionListener(new LookHandler());
		item4.addActionListener(new LookHandler());
		item5.addActionListener(new LookHandler());
		item6.addActionListener(new LookHandler());
		javaItem.addActionListener(new LookHandler());
		mb.add(screen);*/
		
		JMenu help = new JMenu("Help");
		help.setMnemonic(KeyEvent.VK_H);
		JMenuItem aboutspd = new JMenuItem("About SharpDraw");
		aboutspd.setMnemonic(KeyEvent.VK_A); 
		JMenuItem content= new JMenuItem("Content(C)");
		content.setMnemonic(KeyEvent.VK_C); 
		content.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F1,
				ActionEvent.CTRL_MASK));
		help.add(aboutspd);
		//help.add(content);
		mb.add(help);
		aboutspd.addActionListener(new AboutHandler());
		content.addActionListener(new ContentHandler()); // ޴  
	}

	public void makeToolBar() {
		toolBar = new JToolBar();
		clear = new JButton(); // ư 
		open = new JButton();
		save = new JButton();
		print = new JButton();
		undo = new JButton();
		redo = new JButton();
		Insets inset = new java.awt.Insets(INSET, INSET, INSET, INSET);
		clear.setMargin(inset); 
		open.setMargin(inset); 
		save.setMargin(inset); 
		print.setMargin(inset); 
		undo.setMargin(inset); 
		redo.setMargin(inset);
		undo.setEnabled(false);
		redo.setEnabled(false);
		
		numberFormat = java.text.NumberFormat.getIntegerInstance();
	    formatter = new NumberFormatter(numberFormat);
		formatter.setMinimum(new Integer(Constants.RESIZE_MIN));
        formatter.setMaximum(new Integer(Constants.RESIZE_MAX));
        resizeS = new JSlider(Constants.RESIZE_MIN, 
        		Constants.RESIZE_MAX, Constants.RESIZE_DEF);
		resizeT = new JFormattedTextField(formatter);
		resizeT.setColumns(3);
		resizeT.setText("100");
		resizeS.setPreferredSize(new Dimension(100, 22));
		resizeS.setOpaque(false);
		//resizeS.setForeground(Color.blue);
		/*
		resizeBox = new  JComboBox(resizeArr);
		resizeBox.setSelectedIndex(3);*/
		// tooltiptext
		clear.setToolTipText(" ׸");
		open.setToolTipText(" ");
		save.setToolTipText(" ");
		print.setToolTipText(" ");
		undo.setToolTipText("ǵ");
		redo.setToolTipText("ٽý");
		
		// ̺Ʈ 
		clear.addActionListener(new NewHandler());
		open.addActionListener(new OpenHandler());
		save.addActionListener(new SaveHandler());
		print.addActionListener(new PrintHandler());
		undo.addActionListener(new UndoHandler());
		redo.addActionListener(new RedoHandler());
		resizeS.addChangeListener(new ResizeSHandler());
		resizeS.addMouseListener(new ResizeSMouseHandler());
		resizeT.addPropertyChangeListener(new ResizeTHandler());
		
		// ̹ icon
		clearIcon = new ImageIcon(
				getClass().getResource(Constants.ICON_PATH + "clear.gif"));
		openIcon = new ImageIcon(
				getClass().getResource(Constants.ICON_PATH + "open.gif"));
		saveIcon = new ImageIcon(
				getClass().getResource(Constants.ICON_PATH + "save.gif"));
		printIcon = new ImageIcon(
				getClass().getResource(Constants.ICON_PATH + "print.gif"));
		undoIcon = new ImageIcon(
				getClass().getResource(Constants.ICON_PATH + "undo.gif"));
		redoIcon = new ImageIcon(
				getClass().getResource(Constants.ICON_PATH + "redo.gif"));
		
		// ̹ icon
		clear.setIcon(clearIcon);
		open.setIcon(openIcon);
		save.setIcon(saveIcon);
		print.setIcon(printIcon);
		undo.setIcon(undoIcon);
		redo.setIcon(redoIcon);
		
		// ٿ add
		toolBar.setDoubleBuffered(true);
		toolBar.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
		toolBar.add(new JLabel(" "));
		toolBar.add(clear);
		toolBar.add(print);
		toolBar.add(new JLabel("     "));
		toolBar.add(open);
		toolBar.add(save);
		toolBar.add(new JLabel("     "));
		toolBar.add(undo);
		toolBar.add(redo);
		toolBar.add(new JLabel("     "));
		toolBar.add(new JLabel(" Zoom:"));
		toolBar.add(resizeT);
		toolBar.add(new JLabel("% "));
		toolBar.add(resizeS);
		toolBar.add(new JLabel("     "));
		resizeKeyCheck();
		upperToolBox = new UpperToolBox();
	}

	public static void addToolBar() {
		toolBar.add(new JLabel(" "));
		toolBar.add(clear);
		toolBar.add(print);
		toolBar.add(new JLabel("     "));
		toolBar.add(open);
		toolBar.add(save);
		toolBar.add(new JLabel("     "));
		toolBar.add(undo);
		toolBar.add(redo);
		toolBar.add(new JLabel("     "));
		toolBar.add(new JLabel(" Zoom:"));
		toolBar.add(resizeT);
		toolBar.add(new JLabel("% "));
		toolBar.add(resizeS);
		toolBar.add(new JLabel("     "));
	}
	
	public void setMode(String _mode) {
		mode = _mode;
		statusBar.setText(mode);
	}

	public void setEnableRedoButton(boolean enable) {
		redo.setEnabled(enable);
	}
	
	public void setEnableUndoButton(boolean enable) {
		undo.setEnabled(enable);
	}
	JViewport scrollerViewPort;
	GridBagLayout gb;
	public void newCanvas(String fileName, int _width, int _height) {		
		panelSizeDimension = new Dimension(_width, _height);
		panel = null;
		panel = new DrawingPanel(panelSizeDimension);
		panel.setFileName(fileName);
		inFrame = new JInternalFrame(fileName, true, true, true, true);
		inFrame.setDefaultCloseOperation(
				JInternalFrame.DO_NOTHING_ON_CLOSE);
		inFrame.setFrameIcon(frameIcon);
		inFrame.setDoubleBuffered(true);
		arrDrawPanel.add(panel);
		arrInFrame.add(inFrame);
		inFrameCount = arrInFrame.size()-1;
		if (inFrameCount < 10) {
			inFrame.setBounds(10 + inFrameLayout + inFrameCount * 30,
					10 + inFrameCount * 30, _width + 10, _height + 35);
		} 
		else {
			inFrameCount = inFrameCount % 10;
			inFrameLayout = inFrameLayout + 30;
			inFrame.setBounds(10 + inFrameLayout + inFrameCount * 30,
					10 + inFrameCount * 30, _width + 10, _height + 35);
		}
		scrollerViewPort = new JViewport();		
		gb = new GridBagLayout();			
		scrollerViewPort.setLayout(gb);				
		scrollerViewPort.setView(panel);
		  
		scroller = new JScrollPane(
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scroller.setBackground(Color.LIGHT_GRAY);
		scroller.setWheelScrollingEnabled(true);
		scroller.setDoubleBuffered(true);
		scroller.setViewport(scrollerViewPort);
		
		//scroller.setVerifyInputWhenFocusTarget(true);
		//scroller.setAutoscrolls(true);
		inFrame.setContentPane(scroller);
		//inFrame.getContentPane().add(scroller, BorderLayout.CENTER);

		inFrame.setBackground(Color.GRAY);
		inFrame.addInternalFrameListener(new InFrameHandler());
		inFrame.addComponentListener(new InFrameResizeHandler());
		inFrame.setVisible(true);
		desktop.add(inFrame);
		try {
			inFrame.setSelected(true);
		} catch (PropertyVetoException e) {
			e.printStackTrace();
		}
		getContentPane().add(BorderLayout.CENTER, desktop);
		redo.setEnabled(false);
		undo.setEnabled(false);
		inFrameNum++;
	}
	
	//  
	@SuppressWarnings("unchecked")
	// panel vector getϿ myFile setϰ save޼ ȣ
	private void saveFile() { 
		if( panel != null) {
			Dimension d = new Dimension(panel.getPanelSize());	
			myFile.setCanvasSize(d.width, d.height);
			myFile.setVector(panel.getVector());
			//spd̸ , image DrawPanel save޼ ȣ
			filePath = myFile.saveFile();	
			if (filePath != null) {
				String[] splitFileName = filePath.split("[.]");
				if (splitFileName[1].equals("jpg")) {
					panel.save(filePath, d.width, d.height);
				} else if (splitFileName[1].equals("gif")) {
					panel.save(filePath, d.width, d.height);
				} else if (splitFileName[1].equals("png")) {
					panel.save(filePath, d.width, d.height);
				}
				panel.setFileName(
						myFile.getFileName() + "." + splitFileName[1]); // Ȯ
				panel.setFilePath(filePath); // Ȯ 
				inFrame.setTitle(myFile.getFileName() + "." + splitFileName[1]);
				panel.setSaved();
				panel.setClosable(true);
			}
		}
	}

	@SuppressWarnings("unchecked")
	private void saveNameFile() {
		String filePath = panel.getFilePath();
		Dimension d = new Dimension(panel.getPanelSize());
		myFile.setCanvasSize(d.width, d.height);
		myFile.setVector(panel.getVector());
		myFile.saveFile(filePath);		
		String[] splitFileName = filePath.split("[.]");
		if (splitFileName[1].equals("jpg")) {
			panel.save(filePath, d.width, d.height);
		}
		if (splitFileName[1].equals("gif")) {
			panel.save(filePath, d.width, d.height);
		}
		if (splitFileName[1].equals("png")) {
			panel.save(filePath, d.width, d.height);
		}
		panel.setSaved();
		panel.setClosable(true);
	}
	@SuppressWarnings("unchecked")
	private void openFile() {
		save.setEnabled(true);
		Vector<Element> tempDrawVector = new Vector<Element>();
		boolean access = myFile.openFile();
		String ext = myFile.getExt();
		// InternalFrame myFile vector getϿ panel set
		
		if (access == true && ext != null) { 
			if (ext.equalsIgnoreCase("jpg") 
					|| ext.equalsIgnoreCase("gif") 
					|| ext.equalsIgnoreCase("png")) {
				Image img = Toolkit.getDefaultToolkit().getImage(
						myFile.getFilePath());
				inFrameHeight = img.getHeight(this);
				inFrameWidth = img.getWidth(this);
				makeNewCanvas(myFile.getFileName(), 
						inFrameWidth, inFrameHeight); // inFrame
				//panel.initFirstLayer();
				Element element = new ImageTool(myFile.getFilePath());
				panel.addFirstElement(element);
				//panel.initCurrentElementInform();
				vectorLayerDialog.setPanelSize(
						panel.getWidth(), panel.getHeight());
				//vectorLayerDialog.addVector(element);
				vectorLayerDialog.invalidate();
				vectorLayerDialog.validate();
				vectorLayerDialog.repaint();
				panel.repaintAll();
			}
			else if (ext.equalsIgnoreCase("spd")) {
				inFrameWidth = myFile.getCanvasWidth();
				inFrameHeight = myFile.getCanvasHeight();
				newCanvas(myFile.getFileName(), inFrameWidth, inFrameHeight);
				tempDrawVector = myFile.getVector();
				panel.setVector(tempDrawVector);
				panel.initCurrentElementInform();
				vectorLayerDialog.setPanelSize(
						panel.getWidth(), panel.getHeight());
				vectorLayerDialog.setVector(tempDrawVector);
				vectorLayerDialog.setLayerSelectByElemNum(tempDrawVector.size()-1);
				panel.repaintAll();
			}
			else if (ext.equalsIgnoreCase("svg")) {
				tempDrawVector = myFile.getVector();
				panel.setVector(tempDrawVector);
				panel.initCurrentElementInform();
				vectorLayerDialog.setPanelSize(
						panel.getPanelWidth(), panel.getPanelHeight());
				vectorLayerDialog.setVector(tempDrawVector);
				vectorLayerDialog.setLayerSelectByElemNum(tempDrawVector.size()-1);
				panel.repaintAll();
			}
			//System.out.println(myFile.getFilePath());
			
			//System.out.println(myFile.getFileName() + "p;" + myFile.getFilePath());
			panel.setFilePath(myFile.getFilePath());
			panel.setSaved();
			panel.setClosable(true);
		}
	}
	public boolean checkExistFileName(String title) {
		for(int i=0; i<arrInFrame.size(); i++) {
			if(arrInFrame.get(i).getTitle().equalsIgnoreCase(title)) {
				try {
					arrInFrame.get(i).setSelected(true);
				} catch (PropertyVetoException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				return true;
			}
		}
		return false;
	}
	
	//make canvas
	public void makeNewCanvas(
			String title, int _widthPixel, int _heightPixel) {
		newCanvas(title, _widthPixel, _heightPixel);
		panel.initFirstLayer();
	}
	
	//using svg open
	public void makeNewCanvas() {
		inFrameWidth = myFile.getCanvasWidth();
		inFrameHeight = myFile.getCanvasHeight();
		newCanvas(myFile.getFileName(), 
				inFrameWidth, inFrameHeight);
	}	
	
	class InFrameResizeHandler extends ComponentAdapter {	
		public void componentHidden (ComponentEvent  e) {
			System.out.println("hidden");			
		}

		public void componentResized(ComponentEvent e) {
			System.out.println("resized");
			/*
			float inFrameWidth = 
				(float)inFrame.getSize().getWidth();
			float inFrameHeight = 
				(float)inFrame.getSize().getHeight()-35;
			float panelWidth = panel.getWidth();
			float panelHeight = panel.getHeight();
			if(inFrameWidth<panelWidth && inFrameHeight < panelHeight)
			panel.setLocation(
					Math.round((inFrameWidth-10-panelWidth)/2),
					Math.round((inFrameHeight-panelHeight)/2));
			panel.revalidate();*/
		}
		
		public void componentShown(ComponentEvent e) {
			System.out.println("shown");
		}
		
		public void componentMoved(ComponentEvent e) {
			//inFrame.setIgnoreRepaint(true);
			//System.out.println("moved");		
		}
	}

	public JSlider getResizeSlider() {
		return resizeS;
	}
	class InFrameHandler extends InternalFrameAdapter { 
		public void internalFrameOpened(InternalFrameEvent e) {
			inFrame.requestFocusInWindow();
		}
		public void internalFrameClosing(InternalFrameEvent e) {
			inFrame = e.getInternalFrame();
			panel = arrDrawPanel.get(arrInFrame.indexOf(inFrame));
			if(panel != null) {
				if (panel.getClosable() == true) {
					inFrame.dispose();
				} 
				else {
					try {
						int selectOption = JOptionPane.showInternalConfirmDialog(
								inFrame, inFrame.getTitle() + "   ұ?",
								"SharpDraw", JOptionPane.YES_NO_CANCEL_OPTION,
								JOptionPane.QUESTION_MESSAGE);
						if (JOptionPane.YES_OPTION == selectOption) {
							saveFile();
							if (filePath != null)
								inFrame.dispose();
						} else if (JOptionPane.NO_OPTION == selectOption) {
							inFrame.dispose();
						}
					} catch (RuntimeException ex) {
					}
				}
			}
		}
		public void internalFrameClosed(InternalFrameEvent e) {
			arrDrawPanel.remove(panel);
			arrInFrame.remove(inFrame);
			inFrame = null;
			panel = null;
			//System.out.println(arrDrawPanel.size() + " " +  arrInFrame.size());
			vectorLayerDialog.initVectorDialog();
		}
		@SuppressWarnings("unchecked")
		public void internalFrameActivated(InternalFrameEvent e) {
			int size = arrInFrame.size();		// System.out.println("stop" + size);
			for (int i = 0; i < size; i++) {
				if (arrDrawPanel.get(i).getFileName().equalsIgnoreCase(
						e.getInternalFrame().getTitle())) {
					inFrame = arrInFrame.get(i);
					panel = arrDrawPanel.get(i);
					if(panel.getDrawingTool() == DrawToolConst.ROUNDRECT) {
						UpperToolBox.drawRoundRectTool();
					} else if(panel.getDrawingTool() == DrawToolConst.TEXT) {
						UpperToolBox.textTool();
					} else {
						UpperToolBox.drawTool();
					}
					UpperToolBox.setStrokeSize(panel.getStrokeSize());
					UpperToolBox.setRoundRectArcWidth(panel.getRoundRectArcWidth());
					UpperToolBox.setRoundRectArcHeight(panel.getRoundRectArcHeight());
					resizeS.setValue(
							Math.round(panel.getCanvasZoomValue()*100));
					vectorLayerDialog.setPanelSize(
							panel.getWidth(), panel.getHeight()); // not problem
					vectorLayerDialog.setVector(panel.getVector());
					WestToolBox.getInstance().setFillColor(
							panel.isColorMode(),
							panel.getFillColor());
					WestToolBox.getInstance().setStrokeColor(
							panel.isStrokeMode(),
							panel.getStrokeColor());
					break;
				}
			}
			System.out.println("activated");
		}
		public void internalFrameDeactivated (InternalFrameEvent e) {
			//System.out.println(" e ");
		}
	}
	
	class AboutHandler implements ActionListener { 
		public void actionPerformed(ActionEvent event) {
			/*
			ImageIcon aboutIcon = new ImageIcon(
					getClass().getResource(Constants.ICON_PATH + "clear.gif"));
			JOptionPane.showMessageDialog(null, 
					" SharpDraw\n\n"
					+ " <Staff>\n"
					+ "  Graphic Tool: Hee-byung Lee\n" 
					+ "  SVG Parser: Hwa-young Son",
					"SharpDraw", JOptionPane.PLAIN_MESSAGE, aboutIcon);*/
			
			InfoDialog infoDialog = InfoDialog.getInstance();
			infoDialog.setVisible(true);
		}
	}
	
	class ContentHandler implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			//Help help = new Help();
		}
	}
	
	class NewHandler implements ActionListener { // new
		public void actionPerformed(ActionEvent event) {
			newFile = NewFileDialog.getInstance();
			newFile.init( "Untitled-" + Integer.toString(inFrameNum));
		}
	}
	
	class OpenHandler implements ActionListener { // open
		public void actionPerformed(ActionEvent event) {
			openFile();
		}
	}
	
	class SaveHandler implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			if (panel != null) {
				// ̹   ƴϸ  
				if (panel.getSaved() == false) { 
					saveFile();
				} else {
					saveNameFile();
				}
			}
		}
	}
	
	class SaveAsHandler implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			saveFile();
		}
	}
	
	class PreviewHandler implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			if (panel != null) {
				PreviewDialog previewDialog = PreviewDialog.getInstance();
				previewDialog.setCanvasSize(panel.getPanelSize());
			}
		}
	}
		
	public void printMethod(PageFormat _pf) {
		try {
			if(job==null) {
				job = PrinterJob.getPrinterJob();		
			}
			pf = _pf;
			if(pf!=null) {		
				//System.out.println(pf.getWidth() + " " + pf.getHeight() + " " + " " + 
	        	//		pf.getImageableWidth() + " " + pf.getImageableHeight() + " " +
	        	//		pf.getImageableX()  + " " + pf.getImageableY());
				job.setPrintable(panel, pf);
			}
			else
				job.setPrintable(panel);
			
			if(job.printDialog()) {
				job.print();
			}
		}
		catch(PrinterException e) {
			e.printStackTrace();
		}
	}
	
	class PrintHandler implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			if(panel != null) {
				if(pf != null)
					printMethod(pf);
				else
					printMethod(null);
			}
		}
	}
	
	private void confirmExit() { // exit     Ȯ
		boolean exitProgram = true;
		int arrInf = arrInFrame.size();
		for (int i = arrInf - 1; i >= 0; i--) {
			inFrame = arrInFrame.get(i);
			panel = arrDrawPanel.get(i);
			if (panel.getClosable() == true) {
				inFrame.dispose();
			} else {
				try {
					int selectOption = JOptionPane.showInternalConfirmDialog(
							inFrame, inFrame.getTitle() + "   ұ?",
							"SharpDraw", JOptionPane.YES_NO_CANCEL_OPTION,
							JOptionPane.QUESTION_MESSAGE);
					if (selectOption == JOptionPane.YES_OPTION) {
						saveFile();
						if (filePath != null) {
							inFrame.dispose();
						}
					} else if (selectOption == JOptionPane.NO_OPTION) {
						inFrame.dispose();
					} else { // cancel̸  
						exitProgram = false; // System.out.println("
												// cancel");
						break;
					}
				} catch (RuntimeException ex) {
				}
			}
		}
		if (exitProgram == true)
			System.exit(0);
	}
	
	class ExitHandler extends WindowAdapter implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			confirmExit();
		}

		public void windowClosing(WindowEvent arg0) {
			confirmExit();
		}
	}	
	
	class ViewHandler implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			if(e.getActionCommand() == "Options") {
				if(((JCheckBoxMenuItem)e.getSource()).isSelected() == true)
					toolBar.setVisible(true);
				else
					toolBar.setVisible(false);
			}
			else if(e.getActionCommand() == "Tools") {
				if(((JCheckBoxMenuItem)e.getSource()).isSelected() == true)
					westToolBox.setVisible(true);
				else
					westToolBox.setVisible(false);
			}
			else if(e.getActionCommand() == "Layers") {
				if(((JCheckBoxMenuItem)e.getSource()).isSelected() == true)
					vectorLayerDialog.setVisible(true);
				else
					vectorLayerDialog.setVisible(false);
			}
		}
	}
	
	class RenderHandler implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			if(panel != null) {
				if(e.getSource().equals(anti)) {
					if(((JCheckBoxMenuItem)e.getSource()).isSelected() == true) {
						panel.setAntiMode(true);
						vectorLayerDialog.setAntiMode(true);	//miniView
					}
					else {
						panel.setAntiMode(false);
						vectorLayerDialog.setAntiMode(false);
					}
				}
				else if(e.getSource().equals(render)) {
					if(((JCheckBoxMenuItem)e.getSource()).isSelected() == true) {
						panel.setRenderMode(true);
					}
					else {
						panel.setRenderMode(false);
					}					
				}
				else if(e.getSource().equals(dither)) {
					if(((JCheckBoxMenuItem)e.getSource()).isSelected() == true) {
						panel.setDitheringMode(true);
					}
					else {
						panel.setDitheringMode(false);
					}
				}
				else if(e.getSource().equals(textAnti)) {
					if(((JCheckBoxMenuItem)e.getSource()).isSelected() == true) {
						panel.setTextAntiMode(true);
					}
					else {
						panel.setTextAntiMode(false);
					}				
				}
				else if(e.getSource().equals(alphaInter)) {
					if(((JCheckBoxMenuItem)e.getSource()).isSelected() == true) {
						panel.setAlphaInterpMode(true);
					}
					else {
						panel.setAlphaInterpMode(false);
					}				
				}
				else if(e.getSource().equals(colorRender)) {
					if(((JCheckBoxMenuItem)e.getSource()).isSelected() == true) {
						panel.setColorRenderMode(true);
					}
					else {
						panel.setColorRenderMode(false);
					}		
				}
				else if(e.getSource().equals(interpolation)) {
					if(((JCheckBoxMenuItem)e.getSource()).isSelected() == true) {
						panel.setInterpolationMode(true);
					}
					else {
						panel.setInterpolationMode(false);
					}		
				}
			}
		}
	}
	
	class RealViewHandler implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			if(((JCheckBoxMenuItem)e.getSource()).isSelected() == true) {
				vectorLayerDialog.setSimpleDrawMode(false);
			}
			else {
				vectorLayerDialog.setSimpleDrawMode(true);
			}
		}
	}
	
	class UndoHandler implements ActionListener { // undo
		public void actionPerformed(ActionEvent event) {
			if(panel != null)
				panel.unDo();
		}
	}

	class RedoHandler implements ActionListener { // redo
		public void actionPerformed(ActionEvent event) {
			if(panel != null)
				panel.reDo();
		}
	}

	class ResizeSHandler implements ChangeListener {
		public void stateChanged(ChangeEvent e) {
			JSlider source = (JSlider)e.getSource();
			resizePercent = (int)source.getValue();
			if(resizePercent>=93 && resizePercent<=115)
				resizePercent=100;
	        if (!source.getValueIsAdjusting()) { //done adjusting
	        	resizeT.setValue(resizePercent);
	        } else { //value is adjusting; just set the text
	        	resizeT.setText(String.valueOf(resizePercent));        	   
	        }
	        if(panel != null) {
		        panel.setResize((float)resizePercent/100.0f);
				int resizeWidth = (int)Math.round(
						panelSizeDimension.getWidth()*resizePercent/100.0);
				int resizeHeight = (int)Math.round(
						panelSizeDimension.getHeight()*resizePercent/100.0);
				inFrame.setPreferredSize(
						new Dimension(resizeWidth, resizeHeight));
				inFrame.validate();
	        }
	        try {
				Thread.sleep(10);
			} catch (InterruptedException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
		}
	}
	
	class ResizeSMouseHandler implements MouseListener {

		public void mouseClicked(MouseEvent arg0) {	}

		public void mouseEntered(MouseEvent arg0) {	}

		public void mouseExited(MouseEvent arg0) {	}

		public void mousePressed(MouseEvent arg0) {
			//if(panel != null)
			//	panel.setZoomingMode(true);		
		}

		public void mouseReleased(MouseEvent arg0) {
			//if(panel != null)
			//	panel.setZoomingMode(false);
		}
		
	}
	
	class ResizeTHandler implements PropertyChangeListener {
		public void propertyChange(PropertyChangeEvent e) {
			if("value".equals(e.getPropertyName())) {	
				Number value = (Number)e.getNewValue();
		        if (resizeT != null && value != null) {
		        	resizeS.setValue(value.intValue());
		        }
			}			
		}
	}
	private void resizeKeyCheck() {
		resizeT.getInputMap().put(KeyStroke.getKeyStroke(
	            KeyEvent.VK_ENTER, 0),
	            "check");
		resizeT.getActionMap().put("check", new AbstractAction() {
			private static final long serialVersionUID = 1599474983247483915L;
			public void actionPerformed(ActionEvent e) {
				if (!resizeT.isEditValid()) { //The text is invalid.
					Toolkit.getDefaultToolkit().beep();
					resizeT.selectAll();
				} else try {                    //The text is valid,
					resizeT.commitEdit();     //so use it.
				} catch (java.text.ParseException exc) { }
			}
		});
	}

	public UpperToolBox getUpperToolBox() {
		return upperToolBox;
	}
}