package com.sharppaint.drawtools;

import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import javax.swing.ImageIcon;

public class MultiLineTool extends GradientHelper {
	/**
	 * 
	 */
	private static final long serialVersionUID = -6646633853021089752L;

	/********************** never change *********************/
	private boolean closePath = false;
	private AffineTransform atMulti, saveMult;
	transient private Rectangle bound;
	/********************** never change *********************/
	
	public MultiLineTool(float px, float py) {
		toolName = DrawToolConst.MULTILINE;
		path = new GeneralPath();
		x1 = px;
		y1 = py;
		path.moveTo(x1, y1);
		atMulti = new AffineTransform();
		saveMult = new AffineTransform();
		angle = 0;
		rotateMode = false;		
		points = new PointF[1200];
		points[0] = new PointF(x1, y1);
		pointCount++;
		//prevPoint = new Point();
		//prevprevPoint = new Point();
	}
	public MultiLineTool(Element e) {
		toolName = DrawToolConst.MULTILINE;
		closePath = ((MultiLineTool)e).closePath;
		path = (GeneralPath) e.path.clone();
		atMulti = new AffineTransform();
		saveMult = new AffineTransform();
		x1=e.x1;
		y1=e.y1;
		colorMode = e.colorMode;
		gradientMode = e.gradientMode;
		patternMode = e.patternMode;
		strokeMode = e.strokeMode;
		
		angle = e.angle;
		rotateMode = e.rotateMode;
		scaleMode = e.scaleMode;
		scaleX = e.scaleX;
		scaleY = e.scaleY;
		selectStroke = e.selectStroke;
		if(colorMode) {
			colorOpacity = e.colorOpacity;
			fillColor = e.fillColor;
		}
		if(gradientMode) {		//gradList = e.gradList;		
			gradientOpacity = e.gradientOpacity;
			scaleGrad = e.scaleGrad;
			angleGrad = e.angleGrad;	
			gradStyle = e.gradStyle;
			isGradCycle = e.isGradCycle;
			gradList = new ArrayList<GradientTool>();
			for(int i=0; i<e.gradList.size(); i++) {
				gradRect = new GradientTool(e.gradList.get(i));
				gradList.add(gradRect);
			}
			initGradient();
		}
		if(patternMode) {
			patternPath = e.patternPath;
			patternScale = e.patternScale;
			patternOpacity = e.patternOpacity;
			patternImg = new ImageIcon(
					getClass().getResource(patternPath)).getImage();
			patternSrcWidth = e.patternSrcWidth;
			patternSrcHeight = e.patternSrcHeight;
			patternScaleWidth = e.patternScaleWidth;
		}
		if(strokeMode) {
			strokeSize = e.strokeSize;
			strokeOpacity = e.strokeOpacity;
			strokePosition = e.strokePosition;
			strokeFillSelector = e.strokeFillSelector;
			if(strokeFillSelector == 0) {
				strokeColor = e.strokeColor;
			}
			else if(strokeFillSelector == 1) {
				strokeGradScale = e.strokeGradScale;
				strokeGradAngle = e.strokeGradAngle;
				strokeGradStyle = e.strokeGradStyle;
				isStrokeGradCycle = e.isStrokeGradCycle;
				strokeGradList = new ArrayList<GradientTool>();
				for(int i=0; i<e.strokeGradList.size(); i++) {
					gradRect = new GradientTool(e.strokeGradList.get(i));
					strokeGradList.add(gradRect);
				}
				initStrokeGradient();
			}
			else if(strokeFillSelector == 2) {
				strokePatternScale = e.strokePatternScale;
				strokePatternPath = e.strokePatternPath;
				strokeImg = new ImageIcon(
						getClass().getResource(strokePatternPath)).getImage();
				strokeImgWidth = e.strokeImgWidth;
				strokeImgHeight = e.strokeImgHeight;
				strokePatternScaleWidth = e.strokePatternScaleWidth;
			}
		}
		selectStroke = e.selectStroke;
		points = new PointF[1200];
		points = e.points;
		pointCount = e.pointCount;
		setCopyPoint(e.points);
	}/*
	public void initAffineTransform() {
		if (atMulti == null)
			atMulti = new AffineTransform();
		if (saveMult == null)
			saveMult = new AffineTransform();
	}*/
	
	public void setPoint(PointF point){
		points[pointCount] = point;
		pointCount++;
	}
	
	public void setPoint(float x, float y){
		points[pointCount] = new PointF(x, y);
		pointCount++;
	}

	public void setCopyPoint(PointF[] p){
		for(int i=0; i<pointCount; i++) {
			points[i].setX(p[i].getX());
			points[i].setY(p[i].getY());
		}
	}
	
	public void setMultiLineClose(){
		closePath = true;		//
		path.closePath();
	}
	
	public boolean isClosePath() {
		return closePath;
	}
	
	public void setPathfromPoint() {
		path = new GeneralPath();
		path.moveTo(points[0].x, points[0].y);
		for ( int i=1; i < pointCount; i++ ){
			path.lineTo(points[i].x, points[i].y);        
		}
		if(closePath==true) {
			path.closePath();
		}
		setMultiLineBounds();
	}
	
	public void setPathPoint(float px, float py){
		path.lineTo(px, py);
	}
	
	public void setMultiLineBounds() {
		bound = path.getBounds();
		x1 = (float)bound.getMinX();
		y1 = (float)bound.getMinY();	
		width = (float)bound.getWidth();
		height = (float)bound.getHeight();
	}
	
	public boolean match(float x, float y) {
		match = false;
		if(path.contains(x, y))
			match = true;
		return match;
	}	
	
	public void translate(float _x1, float _y1, float _x2, float _y2) {
		atMulti.setToTranslation(_x2-_x1, _y2-_y1);
		path.transform(atMulti);
		bound = path.getBounds();
		x1 = (float)bound.getMinX();
		y1 = (float)bound.getMinY();	
		width = (float)bound.getWidth();
		height = (float)bound.getHeight();;
		for(int i=0;i<pointCount;i++){
			points[i].translate(_x2-_x1, _y2-_y1);
		}
	}
	
	public PointF[] getPoints() {
		return points;
	}
	
	public void simpleDraw(Graphics g, float scale) {
		Graphics2D g2d = (Graphics2D) g;
		AffineTransform ats = new AffineTransform();	
		AffineTransform saveforms = g2d.getTransform();	
		ats.scale(scale, scale);
		g2d.transform(ats);
		g2d.setStroke(new BasicStroke(5));
		if(rotateMode || scaleMode) {
			at = new AffineTransform();	
			saveform = g2d.getTransform();		
			at.setToTranslation(x1 + width/2, y1 + height/2);
			saveMult.setToTranslation(-x1-width/2, -y1-height/2);
			if(rotateMode) { 
				at.rotate(Math.toRadians(angle)); 
			}
			if(scaleMode) {
				at.scale(scaleX, scaleY);
			}
			g2d.transform(at); 
			g2d.fill(saveMult.createTransformedShape(path));
			g2d.setColor(Color.BLACK);
			g2d.draw(saveMult.createTransformedShape(path));
			g2d.setTransform(saveform);
		}
		else {	//not rotate, not resize mode
			g2d.setColor(Color.BLACK);
			g2d.draw(path);
		}
		g2d.setTransform(saveforms);
	}
	public void draw(Graphics g) {
		Graphics2D g2d = (Graphics2D) g;		
		g2d.setStroke(new BasicStroke(strokeSize));
		if(hide == false) {
			if(rotateMode || scaleMode) {
				at = new AffineTransform();
				saveform = g2d.getTransform();	
				at.setToTranslation(x1 + width/2, y1 + height/2);
				//AffineTransform saveMult = new AffineTransform();
				saveMult.setToTranslation(-x1-width/2, -y1-height/2);
				if(rotateMode) { 
					at.rotate(Math.toRadians(angle)); 
				}
				if(scaleMode) {
					at.scale(scaleX, scaleY);
				}
				g2d.transform(at); 				
				
				if(colorMode) {
					AlphaComposite ac = AlphaComposite.getInstance(
							AlphaComposite.SRC_OVER, colorOpacity);
					g2d.setComposite(ac);
					g2d.setColor(fillColor);
					g2d.fill(saveMult.createTransformedShape(path));
				}					
				if(gradientMode) {
					if(width!=0) {
						drawGradientTransform(g2d);
						g2d.fill(saveMult.createTransformedShape(path));
					}					
				}
				if(patternMode) {
					if(firstPatternMode) {
						bi = new BufferedImage(
								patternScaleWidth, patternScaleWidth, 
								BufferedImage.TYPE_INT_RGB);
						big = bi.createGraphics();
						big.drawImage(patternImg, 0, 0, 
								patternScaleWidth, patternScaleWidth, 
								0, 0, patternSrcWidth, patternSrcHeight, null);
						texture = new TexturePaint(bi, 
								new Rectangle2D.Float(
										-width/2, -height/2, 
										patternScaleWidth, patternScaleWidth));
						AlphaComposite pac = AlphaComposite.getInstance(
								AlphaComposite.SRC_OVER, patternOpacity);
						g2d.setComposite(pac);
						g2d.setPaint(texture);						
						g2d.fill(saveMult.createTransformedShape(path));
						firstPatternMode = false;
					}
					else {
						AlphaComposite pac = AlphaComposite.getInstance(
								AlphaComposite.SRC_OVER, patternOpacity);
						g2d.setComposite(pac);
						g2d.setPaint(texture);	
						g2d.fill(saveMult.createTransformedShape(path));
					}
				}
				if(strokeMode) {	//strokeMode
					g2d.setStroke(new BasicStroke(strokeSize));
					AlphaComposite ac = AlphaComposite.getInstance(
							AlphaComposite.SRC_OVER, strokeOpacity);
					g2d.setComposite(ac);
					if(strokeFillSelector == 0) {	
						g2d.setColor(strokeColor);
					}
					else if(strokeFillSelector == 1) {	//gradient
						if(width!=0) {
							drawStrokeGradientTransform(g2d);
						}
					}		
					else if(strokeFillSelector == 2) {	//pattern
						if(firstStrokePatternMode) {
							BufferedImage bii = new BufferedImage(
									strokePatternScaleWidth, strokePatternScaleWidth, 
									BufferedImage.TYPE_INT_RGB);
							Graphics2D biig = bii.createGraphics();
							biig.drawImage(strokeImg, 0, 0, 
									strokePatternScaleWidth, strokePatternScaleWidth, 
									0, 0, strokeImgWidth, 
									strokeImgHeight, null);
							strokeTexture = new TexturePaint(bii, 
									new Rectangle2D.Float(
											-width/2, -height/2, 
											strokePatternScaleWidth, 
											strokePatternScaleWidth));		
							firstStrokePatternMode = false;
						}
						g2d.setPaint(strokeTexture);
					}							
					if(strokePosition == 0) {	//outside
						g2d.draw(saveMult.createTransformedShape(path));
					}
					else if(strokePosition == 1) {	//inside
						g2d.draw(saveMult.createTransformedShape(path));
					}
					else if(strokePosition == 2) {	//center
						g2d.draw(saveMult.createTransformedShape(path));
					}
				}		
				if(select) {
					if(selectStroke == null)
						selectStroke = new BasicStroke(1);
					g2d.setStroke(selectStroke);
					g2d.setColor(Color.BLUE);
					g2d.draw(saveMult.createTransformedShape(path));
				} else if(reShape) {
					drawReshapeStroke(g2d, rotateMode || scaleMode);				
				}
				g2d.setTransform(saveform);
			}
			else {	//not rotate, not resize mode
				if(colorMode) {
					AlphaComposite ac = AlphaComposite.getInstance(
							AlphaComposite.SRC_OVER, colorOpacity);
					g2d.setComposite(ac);
					g2d.setColor(fillColor);
					g2d.fill(path);
				}
				if(gradientMode)  {
					if(width != 0) {
						drawGradient(g2d);
						g2d.fill(path);
					}
				}			
				if(patternMode) {
					if(firstPatternMode) {						
						bi = new BufferedImage(
								patternScaleWidth, patternScaleWidth, 
								BufferedImage.TYPE_INT_RGB);
						big = bi.createGraphics();
						big.drawImage(patternImg, 0, 0, 
								patternScaleWidth, patternScaleWidth, 
								0, 0, patternSrcWidth, patternSrcHeight, null);
						texture = new TexturePaint(bi, 
								new Rectangle2D.Float(
								x1, y1, 
								patternScaleWidth, 
								patternScaleWidth));
						AlphaComposite pac = AlphaComposite.getInstance(
								AlphaComposite.SRC_OVER, patternOpacity);
					g2d.setComposite(pac);
						g2d.setPaint(texture);						
						g2d.fill(path);
						firstPatternMode = false;
					}
					else {
						AlphaComposite pac = AlphaComposite.getInstance(
								AlphaComposite.SRC_OVER, patternOpacity);
						g2d.setComposite(pac);
						g2d.setPaint(texture);
						g2d.fill(path);
					}
				}
				if(strokeMode) {	//strokeMode
					//g2d.setStroke(new BasicStroke(strokeSize));
					g2d.setStroke(new BasicStroke(strokeSize));
					AlphaComposite ac = AlphaComposite.getInstance(
							AlphaComposite.SRC_OVER, strokeOpacity);
					g2d.setComposite(ac);
					if(strokeFillSelector == 0) {	
						g2d.setColor(strokeColor);
					}
					else if(strokeFillSelector == 1) {	//gradient
						if(width != 0) {
							drawStrokeGradient(g2d);
						}
					}
					else if(strokeFillSelector == 2) {	//pattern
						if(firstStrokePatternMode) {
							BufferedImage bii = new BufferedImage(
									strokePatternScaleWidth, strokePatternScaleWidth, 
									BufferedImage.TYPE_INT_RGB);
							Graphics2D biig = bii.createGraphics();
							biig.drawImage(strokeImg, 0, 0, 
									strokePatternScaleWidth, strokePatternScaleWidth, 
									0, 0, strokeImgWidth, 
									strokeImgHeight, null);
							strokeTexture = new TexturePaint(bii, 
									new Rectangle2D.Float(
											x1, y1, 
											strokePatternScaleWidth, 
											strokePatternScaleWidth));
							firstStrokePatternMode = false;
						}
						g2d.setPaint(strokeTexture);							
					}				
					
					if(strokePosition == 0) {	//outside
						g2d.draw(path);
					}
					else if(strokePosition == 1) {	//inside
						g2d.draw(path);
					}
					else if(strokePosition == 2) {	//center
						g2d.draw(path);
					}
				}			
				if(select) {
					if(selectStroke == null)
						selectStroke = new BasicStroke(1);
					g2d.setStroke(selectStroke);
					g2d.setColor(Color.BLUE);
					g2d.draw(path);
				} else if(reShape) {
					drawReshapeStroke(g2d, rotateMode || scaleMode);				
				}
			}	
		}
	}
	public void smallDraw(Graphics g, float scale) {
		Graphics2D g2d = (Graphics2D) g;
		AffineTransform ats = new AffineTransform();	
		AffineTransform saveforms = g2d.getTransform();	
		ats.scale(scale, scale);
		g2d.transform(ats);
		
		if(rotateMode || scaleMode) {
			at = new AffineTransform();
			at.setToTranslation(x1 + width/2, y1 + height/2);
			//AffineTransform saveMult = new AffineTransform();
			saveMult.setToTranslation(-x1-width/2, -y1-height/2);
			if(rotateMode) { 
				at.rotate(Math.toRadians(angle)); 
			}
			if(scaleMode) {
				at.scale(scaleX, scaleY);
			}
			g2d.transform(at); 
			if(colorMode) {
				AlphaComposite ac = AlphaComposite.getInstance(
						AlphaComposite.SRC_OVER, colorOpacity);
				g2d.setComposite(ac);
				g2d.setColor(fillColor);
				g2d.fill(saveMult.createTransformedShape(path));
			}					
			if(gradientMode) {
				if(width!=0) {
					drawGradientTransform(g2d);
					g2d.fill(saveMult.createTransformedShape(path));
				}
			}
			if(patternMode) {			
				AlphaComposite pac = AlphaComposite.getInstance(
						AlphaComposite.SRC_OVER, patternOpacity);
				g2d.setComposite(pac);
				g2d.setPaint(texture);
				g2d.fill(saveMult.createTransformedShape(path));	
			}
			if(strokeMode) {	//strokeMode
				g2d.setStroke(new BasicStroke(strokeSize));
				AlphaComposite ac = AlphaComposite.getInstance(
						AlphaComposite.SRC_OVER, strokeOpacity);
				g2d.setComposite(ac);
				if(strokeFillSelector == 0) {	
					g2d.setColor(strokeColor);
				}
				else if(strokeFillSelector == 1) {	//gradient
					//initStrokeGradient();
					if(width!=0) {
						drawStrokeGradientTransform(g2d);
					}
				}
				else if(strokeFillSelector == 2) {	//pattern
					g2d.setPaint(strokeTexture);			
				}					
				if(strokePosition == 0) {	//outside
					g2d.draw(saveMult.createTransformedShape(path));
				}
				else if(strokePosition == 1) {	//inside
					g2d.draw(saveMult.createTransformedShape(path));
				}
				else if(strokePosition == 2) {	//center
					g2d.draw(saveMult.createTransformedShape(path));
				}
			}
		}
		else {	//not rotate, not resize mode
			if(colorMode) {
				AlphaComposite ac = AlphaComposite.getInstance(
						AlphaComposite.SRC_OVER, colorOpacity);
				g2d.setComposite(ac);
				g2d.setColor(fillColor);
				g2d.fill(path);
			}
			if(gradientMode)  {	
				if(width!=0) {
					drawGradient(g2d);
					g2d.fill(path);
				}
			}			
			if(patternMode) {
				AlphaComposite pac = AlphaComposite.getInstance(
						AlphaComposite.SRC_OVER, patternOpacity);
				g2d.setComposite(pac);
				g2d.setPaint(texture);
				g2d.fill(path);
			}
			if(strokeMode) {	//strokeMode
				//g2d.setStroke(new BasicStroke(strokeSize));
				g2d.setStroke(new BasicStroke(strokeSize));
				AlphaComposite ac = AlphaComposite.getInstance(
						AlphaComposite.SRC_OVER, strokeOpacity);
				g2d.setComposite(ac);
				if(strokeFillSelector == 0) {	
					g2d.setColor(strokeColor);
				}
				else if(strokeFillSelector == 1) {	//gradient
					if(width!=0) {
						drawStrokeGradient(g2d);
					}
				}
				else if(strokeFillSelector == 2) {	//pattern
					g2d.setPaint(strokeTexture);			
				}					
				if(strokePosition == 0) {	//outside
					g2d.draw(path);
				}
				else if(strokePosition == 1) {	//inside
					g2d.draw(path);
				}
				else if(strokePosition == 2) {	//center
					g2d.draw(path);
				}
			}
		}	//not rotate block end
		g2d.setTransform(saveforms);
	}
}
