package com.sharppaint.graphicprocess;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.LinearGradientPaint;
import java.awt.MultipleGradientPaint;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.sharppaint.MainWindow;
import com.sharppaint.drawtools.DrawToolConst;
import com.sharppaint.drawtools.Element;
import com.sharppaint.drawtools.GradientTool;

public class VectorGradientPanel extends VectorBasicData {

	private JPanel gradientPanel, centerPanel;

	private JSlider gradScaleS, gradAngleS, gradOpacityS;

	private JLabel gradScaleL;

	private JLabel gradAngleL, gradPerL, gradRadL;

	private JLabel gradOpacityL, gradOpPerL;

	private JFormattedTextField gradScaleT, gradAngleT, gradOpacityT;

	private int x1, y1, x2;

	private ArrayList<GradientTool> gradList, posGradList;

	private GradientTool gradRect, paintRect, tempRect;

	private boolean match = false;

	private GradPanel gradPanel;

	private GradClickPanel gradClickPanel;

	private Color gradButtonColor = Color.WHITE;

	private Color tempColor;

	private float scaleGrad = 100.0f, angleGrad = 0f;

	private int gradientOpacity = 100;

	private Color[] gradColors;
	
	private float[] gradDist;
	
	private int gradListSize;
	
	private static VectorGradientPanel vectorGradientPanel = null;

	public static VectorGradientPanel getInstance(Element elem, int elemNum) {
		element = elem;
		elementNum = elemNum;
		if (vectorGradientPanel == null)
			vectorGradientPanel = new VectorGradientPanel();
		return vectorGradientPanel;
	}

	public static VectorGradientPanel getInstance() {
		return vectorGradientPanel;
	}
	private JRadioButton gradLinearR;
	private JRadioButton gradRadialR;
	private JLabel gradStyleL;
	private GradStyleRadioListener gradStyleRadioListener;
	
	private JRadioButton gradRepeatR;
	private JRadioButton gradNoCycleR;
	private JLabel gradCycleL;
	private GradCycleRadioListener gradCycleRadioListener;
	
	private VectorGradientPanel() {
		centerPanel = new JPanel();
		gradientPanel = new JPanel();
		gradientPanel.setBorder(BorderFactory.createTitledBorder(loweredetched,
				" Gradient "));
		gradientPanel.setPreferredSize(new Dimension(PANEL_WIDTH, 232));

		gradOpacityS = new JSlider(OPACITY_MIN, OPACITY_MAX, OPACITY_MAX);
		gradOpacityS.setPreferredSize(new Dimension(120, 18));
		gradOpacityS.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				gradOpacityS_stateChanged(e);
			}
		});
		gradScaleS = new JSlider(RESIZE_MIN, GRADSCALE_MAX, GRADSCALE_INIT);
		gradScaleS.setPreferredSize(new Dimension(120, 18));
		gradScaleS.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				gradScaleS_stateChanged(e);
			}
		});
		gradAngleS = new JSlider(GRADROTATE_MIN, GRADROTATE_MAX, ZERO);
		gradAngleS.setPreferredSize(new Dimension(120, 18));
		gradAngleS.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				gradAngleS_stateChanged(e);
			}
		});
		formatter.setMinimum(new Integer(OPACITY_MIN));
		formatter.setMaximum(new Integer(OPACITY_MAX));
		gradOpacityT = new JFormattedTextField(formatter);
		gradOpacityT.setValue(new Integer(OPACITY_MAX));
		gradOpacityT.setColumns(3); //get some space
		gradOpacityT.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				gradOpacityT_propertyChange(e);
			}
		});
		formatter.setMinimum(new Integer(RESIZE_MIN));
		formatter.setMaximum(new Integer(GRADSCALE_MAX));
		gradScaleT = new JFormattedTextField(formatter);
		gradScaleT.setValue(new Integer(GRADSCALE_INIT));
		gradScaleT.setColumns(3); //get some space
		gradScaleT.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				gradScaleT_propertyChange(e);
			}
		});
		formatterRotate.setMinimum(new Integer(GRADROTATE_MIN));
		formatterRotate.setMaximum(new Integer(GRADROTATE_MAX));
		gradAngleT = new JFormattedTextField(formatterRotate);
		gradAngleT.setValue(new Integer(ZERO));
		gradAngleT.setColumns(3); //get some space
		gradAngleT.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				gradAngleT_propertyChange(e);
			}
		});
		gradOpacityL = new JLabel("   Opacity:", SwingConstants.RIGHT);
		gradScaleL = new JLabel("         Scale:", SwingConstants.RIGHT);
		gradAngleL = new JLabel("        Angle:", SwingConstants.RIGHT);
		gradOpacityL.setFont(font);
		gradScaleL.setFont(font);
		gradAngleL.setFont(font);
		gradPanel = new GradPanel();
		gradList = element.getGradList();
		if (gradList == null || gradList.size()<2) {
			gradList = new ArrayList<GradientTool>(); 
			gradRect = new GradientTool(0, Color.BLACK);
			gradList.add(gradRect);
			gradRect = new GradientTool(200, Color.WHITE);
			gradList.add(gradRect);
		} 
		else {
			scaleGrad = element.getGradScalePercent();
			angleGrad = element.getGradAngle();
			gradScaleS.setValue(Math.round(scaleGrad));
			gradAngleT.setValue(Math.round(angleGrad));
			gradientOpacity = element.getGradOpacityPercent();
			gradOpacityS.setValue(gradientOpacity);
		}
		gradClickPanel = new GradClickPanel();
		gradPanel.setPreferredSize(new Dimension(200, 35));
		gradClickPanel.setPreferredSize(new Dimension(210, 13));
		gradClickPanel.setBackground(Color.GRAY);
		gradClickPanel.addMouseListener(new MouseListener());
		gradClickPanel.addMouseMotionListener(new MouseMotionListener());
		gradOpPerL = 	new JLabel("%    ");
		gradPerL = 		new JLabel("%     ");
		gradRadL = 		new JLabel("      ");
		gradOpPerL.setFont(font);
		gradPerL.setFont(font);
		gradRadL.setFont(font);

		gradStyleL = new JLabel("         Style:", SwingConstants.RIGHT);
		gradStyleRadioListener = new GradStyleRadioListener();
		gradLinearR = new JRadioButton("Linear");
		gradLinearR.setActionCommand("Linear");
		gradRadialR = new JRadioButton("Radial");
		gradRadialR.setActionCommand("Radial");
		gradLinearR.addActionListener(gradStyleRadioListener);
		gradRadialR.addActionListener(gradStyleRadioListener);
		gradStyleL.setFont(font);
		gradLinearR.setFont(font);
		gradRadialR.setFont(font);
		gradCycleL = new JLabel("         Cycle:", SwingConstants.RIGHT);
		gradCycleRadioListener = new GradCycleRadioListener();
		gradNoCycleR = new JRadioButton("No_Cycle");
		gradNoCycleR.setActionCommand("No_Cycle");
		gradNoCycleR.addActionListener(gradCycleRadioListener);
		gradRepeatR = new JRadioButton("Repeat");
		gradRepeatR.setActionCommand("Repeat");
		gradRepeatR.addActionListener(gradCycleRadioListener);
		gradCycleL.setFont(font);
		gradNoCycleR.setFont(font);
		gradRepeatR.setFont(font);
		
		gradientKeyCheck();
		gradClickPanel.repaint();
		gradientPanel.add(gradOpacityL);
		gradientPanel.add(gradOpacityS);
		gradientPanel.add(gradOpacityT);
		gradientPanel.add(gradOpPerL);
		//gradientPanel.add(Box.createRigidArea(new Dimension(250, 10)));	
		gradientPanel.add(gradPanel);
		gradientPanel.add(gradClickPanel);
		//gradientPanel.add(Box.createRigidArea(new Dimension(80, 30)));
		gradientPanel.add(gradScaleL);
		gradientPanel.add(gradScaleS);
		gradientPanel.add(gradScaleT);
		gradientPanel.add(gradPerL);
		gradientPanel.add(gradAngleL);
		gradientPanel.add(gradAngleS);
		gradientPanel.add(gradAngleT);
		gradientPanel.add(gradRadL);
	    ButtonGroup gradStyleGroup = new ButtonGroup();
	    gradStyleGroup.add(gradLinearR);
	    gradStyleGroup.add(gradRadialR);
	    gradientPanel.add(gradStyleL);
		gradientPanel.add(gradLinearR);
		gradientPanel.add(gradRadialR);
		gradientPanel.add(new JLabel("                     "));
	    ButtonGroup gradCycleGroup = new ButtonGroup();
	    gradCycleGroup.add(gradNoCycleR);
	    gradCycleGroup.add(gradRepeatR);
		gradientPanel.add(gradCycleL);
		gradientPanel.add(gradNoCycleR);
		gradientPanel.add(gradRepeatR);
		gradientPanel.add(new JLabel("               "));
		
		gradientPanel.setBorder(BorderFactory.createTitledBorder(loweredetched,
				" Gradient "));

		centerPanel.add(gradientPanel);
		centerPanel.add(Box.createRigidArea(new Dimension(0, 128)));
		centerPanel.setBorder(BorderFactory.createTitledBorder(loweredetched,
				" Gradient Overlay "));
	}
	class GradStyleRadioListener implements ActionListener { 
        public void actionPerformed(ActionEvent e) {
        	if(e.getActionCommand().equals("Linear")) {
        		element.setGradStyle(DrawToolConst.LINEAR_GRAD);        		
        		MainWindow.getInstance().getPanel().setGradStyle(
        				element, elementNum,
        				DrawToolConst.LINEAR_GRAD);
        	} else {
        		element.setGradStyle(DrawToolConst.RADIAL_GRAD);     		
        		MainWindow.getInstance().getPanel().setGradStyle(
        				element, elementNum,
        				DrawToolConst.RADIAL_GRAD);        		
        	}
        }
    }
	class GradCycleRadioListener implements ActionListener { 
        public void actionPerformed(ActionEvent e) {
        	if(e.getActionCommand().equals("Repeat")) {
        		element.setGradCycle(true);        		
        		MainWindow.getInstance().getPanel().setGradCycle(
        				element, elementNum, true);
        	} else {
        		element.setGradCycle(false);        		
        		MainWindow.getInstance().getPanel().setGradCycle(
        				element, elementNum, false);	
        	}
        }
    }

	public JPanel getPanel() {
		return centerPanel;
	}

	public void init() {
		gradList = element.getGradList();
		if (gradList == null) {
			gradList = new ArrayList<GradientTool>(); 
			gradRect = new GradientTool(0, Color.BLACK);
			gradList.add(gradRect);
			gradRect = new GradientTool(200, Color.WHITE);
			gradList.add(gradRect);
			posGradList = (ArrayList<GradientTool>) gradList;
		} else {
			scaleGrad = element.getGradScalePercent();
			angleGrad = element.getGradAngle();
			if (gradScaleS != null)
				gradScaleS.setValue(Math.round(scaleGrad));
			if (gradAngleT != null)
				gradAngleT.setValue(Math.round(angleGrad));
			gradientOpacity = Math.round(element.getGradOpacityPercent());
			gradOpacityS.setValue(gradientOpacity);
			posGradList = (ArrayList<GradientTool>) gradList;
		}
		gradPanel.repaint();
		gradClickPanel.setVisible(true);
		gradPanel.setVisible(true);
		gradScaleL.setEnabled(true);
		gradAngleL.setEnabled(true);
		gradOpacityL.setEnabled(true);
		gradScaleS.setEnabled(true);
		gradAngleS.setEnabled(true);
		gradOpacityS.setEnabled(true);
		gradScaleT.setEnabled(true);
		gradAngleT.setEnabled(true);
		gradOpacityT.setEnabled(true);
		
		if(element.getGradStyle() == DrawToolConst.LINEAR_GRAD) {
			gradLinearR.setSelected(true);
		} else {
			gradRadialR.setSelected(true);
		}
		if(element.isGradCycle()) {
			gradRepeatR.setSelected(true);
		} else {
			gradNoCycleR.setSelected(true);
		}
	}

	@SuppressWarnings("unchecked")
	public void setDisable() {
		gradClickPanel.setVisible(false);
		gradPanel.setVisible(false);
		gradScaleL.setEnabled(false);
		gradAngleL.setEnabled(false);
		gradOpacityL.setEnabled(false);
		gradScaleS.setEnabled(false);
		gradAngleS.setEnabled(false);
		gradOpacityS.setEnabled(false);
		gradScaleT.setEnabled(false);
		gradAngleT.setEnabled(false);
		gradOpacityT.setEnabled(false);
		if (element != null) {
			if (posGradList == null
					&& MainWindow.getInstance().getPanel().getGradList() != null) {
				posGradList = (ArrayList<GradientTool>)
						MainWindow.getInstance().getPanel().getGradList().clone();
			}
			//checkҶ gradient  س  
			element.setGradient(posGradList, gradientOpacity, false);
			MainWindow.getInstance().getPanel().setGradient(element,
					elementNum, scaleGrad, angleGrad, gradientOpacity, false);
		}
	}

	private void gradOpacityS_stateChanged(ChangeEvent e) {
		JSlider source = (JSlider) e.getSource();
		gradientOpacity = (int) source.getValue();
		if (!source.getValueIsAdjusting()) { //done adjusting
			gradOpacityT.setValue(gradientOpacity);
		} else { //value is adjusting; just set the text
			gradOpacityT.setText(String.valueOf(gradientOpacity));
		}
		element.setGradOpacityPercent(gradientOpacity);
		MainWindow.getInstance().getPanel().setGradOpacity(
				element, elementNum,
				gradientOpacity);
	}

	private void gradOpacityT_propertyChange(PropertyChangeEvent e) {
		if ("value".equals(e.getPropertyName())) {
			Number value = (Number) e.getNewValue();
			if (gradOpacityT != null && value != null) {
				gradOpacityS.setValue(value.intValue());
			}
		}
	}

	private void gradScaleS_stateChanged(ChangeEvent e) {
		JSlider source = (JSlider) e.getSource();
		scaleGrad = (float) source.getValue();
		if (!source.getValueIsAdjusting()) { //done adjusting
			gradScaleT.setValue(new Integer(Math.round(scaleGrad)));
		} else { //value is adjusting; just set the text
			gradScaleT.setText(String.valueOf(scaleGrad));
		}
		element.setGradScalePercent(scaleGrad);
		gradPanel.repaint();
		//StartDraw.panel.setResizeX(element, elementNum, percent);
	}

	private void gradAngleS_stateChanged(ChangeEvent e) {
		JSlider source = (JSlider) e.getSource();
		angleGrad = (float) source.getValue();
		if (!source.getValueIsAdjusting()) { //done adjusting        	
			gradAngleT.setValue(new Integer(Math.round(angleGrad)));
		} else { //value is adjusting; just set the text
			gradAngleT.setText(String.valueOf(angleGrad));
		}
		element.setGradAngle(angleGrad);
		gradPanel.repaint();
		//StartDraw.panel.setResizeX(element, elementNum, percent);
	}

	private void gradScaleT_propertyChange(PropertyChangeEvent e) {
		if ("value".equals(e.getPropertyName())) {
			Number value = (Number) e.getNewValue();
			if (gradScaleT != null && value != null) {
				gradScaleS.setValue(value.intValue());
			}
		}
	}

	private void gradAngleT_propertyChange(PropertyChangeEvent e) {
		if ("value".equals(e.getPropertyName())) {
			Number value = (Number) e.getNewValue();
			if (gradAngleT != null && value != null) {
				gradAngleS.setValue(value.intValue());
				System.out.println(value.intValue());
			}
		}
	}

	class GradPanel extends JPanel {
		private static final long serialVersionUID = -439029742615476538L;

		@SuppressWarnings("unchecked")
		public void paintComponent(Graphics g) { // drawVector ׸
			super.paintComponent(g);
			Graphics2D g2d = (Graphics2D) g;
			posGradList = (ArrayList<GradientTool>) gradList.clone();

			for (int pick = 1; pick < posGradList.size(); pick++) {
				int current = pick;
				paintRect = posGradList.get(pick);
				for (; (current > 0)
						&&posGradList.get(current - 1).getX() > paintRect.getX();
						current--) {
					posGradList.set(current, posGradList.get(current - 1));
				}
				posGradList.set(current, paintRect);
			}
			for(int i=0; i<posGradList.size()-1; i++) {
				if(posGradList.get(i).getX() >= posGradList.get(i+1).getX()) {					
					if(posGradList.size()>2) {
						posGradList.remove(i);
						i--;
					}
					else {
						GradientTool tempGRect = posGradList.get(i);
						if(tempGRect.getX()<95)
							posGradList.get(i+1).setX(tempGRect.getX()+1);
						else {
							posGradList.get(i).setX(posGradList.get(i+1).getX()-1);
						}
					}
				}
			}
			boolean isGradListMistaken = false;
			for(int i=0; i<posGradList.size()-1; i++) {
				if(posGradList.get(i).getX() >= posGradList.get(i+1).getX()) {
					isGradListMistaken = true;
					break;
				}
			}
			if (posGradList != null && posGradList.size()>1
					&& !isGradListMistaken) {
				element.setGradient(posGradList, gradientOpacity, true);
				MainWindow.getInstance().getPanel().setGradient(
						element, elementNum,
						scaleGrad, angleGrad,
						gradientOpacity, true);

				gradListSize = posGradList.size();
				gradColors = new Color[gradListSize];
				gradDist = new float[gradListSize];
				
				for(int i=0; i<posGradList.size(); i++) {
					tempRect = posGradList.get(i);
					gradColors[i] = tempRect.getColor();
					gradDist[i] = tempRect.getDist();
				}
				g2d.setPaint(new LinearGradientPaint(
						0, 
						0, 
						198, 
						0, 
						gradDist, gradColors, 
						MultipleGradientPaint.CycleMethod.NO_CYCLE));
				g2d.fillRect(0, 0, 198, 35);
				g2d.setColor(Color.BLACK);
				g2d.drawRect(0, 0, 199, 34);
				g2d.setColor(Color.WHITE);
				g2d.drawRect(1, 1, 197, 32);
				g.dispose();
			}
		}
	}

	class GradClickPanel extends JPanel {
		private static final long serialVersionUID = 5560088132722209183L;

		public void paintComponent(Graphics g) { // drawVector ׸
			super.paintComponent(g); //panel paintComponent
			for (int i = 0; i < gradList.size(); i++) {
				tempRect = gradList.get(i);
				tempRect.draw(g);
			}
			g.dispose();
		}
	}

	class MouseListener extends MouseAdapter {
		public void mousePressed(MouseEvent e) {
			x1 = e.getX();
			y1 = e.getY();
			for (int index = 0; index < gradList.size(); index++) {
				gradRect = gradList.get(index);
				match = gradRect.match(x1, y1);
				if (match == true) {
					break;
				} else {
					gradRect.setSelected(false);
				}
			}
			if (match == true) {
				gradRect.setSelected(true);
				// Ŭ ι Color
				if (e.getButton() == 1 && e.getClickCount() == 2) {
					tempColor = gradButtonColor;
					gradButtonColor = JColorChooser.showDialog(
							MainWindow.getInstance().getPanel(),
							" ", gradButtonColor);
					if (gradButtonColor == null)
						gradButtonColor = tempColor;
					else {
						gradRect.setColor(gradButtonColor);
						//gradVector.removeElementAt(index);		
						//gradVector.add(rect);
						gradPanel.repaint();
						gradClickPanel.repaint();
					}
				}
				// Ŭ ι 
				else if (e.getButton() == 3 && e.getClickCount() == 2) {
					if(gradList.size()>2) {
						gradList.remove(gradRect);
						gradPanel.repaint();
						gradClickPanel.repaint();
					}
				} 
				// Ŭ ƴҽ ̵ 
				//else 
				//	firstMove = true;
			} 
			else { //match   ο point
				if (e.getButton() == 1) {
					if(x1 >= 5 && x1 <= 205) {
						gradRect = new GradientTool(x1 - 5, Color.LIGHT_GRAY);
					}
					else if(x1 < 5) {
						gradRect = new GradientTool(0, Color.LIGHT_GRAY);
					}
					else if(x1 > 205) {
						gradRect = new GradientTool(200, Color.LIGHT_GRAY);
					}
					gradList.add(gradRect);
					gradPanel.repaint();
					gradClickPanel.repaint();
				}
			}
		}

		public void mouseReleased(MouseEvent e) { //ư released
			gradPanel.repaint();
			gradClickPanel.repaint();
		}
	}

	class MouseMotionListener extends MouseMotionAdapter {
		public void mouseDragged(MouseEvent e) {
			x2 = e.getX();
			if (match == true) {
				if (x2 >= 5 && x2 <= 205) { //    5 205					
					gradRect.translate(x1, x2);
				}
				else {
					// ̵
					if(x2 < x1) {
						if(x2 < 5) {
							gradRect.setX(0);
						}							
					}
					// ̵
					else {
						if(x2 > 205) {
							gradRect.setX(200);	
						}			
					}
				}
				x1 = x2;
			}
			gradClickPanel.repaint();
		}

		public void mouseMoved(MouseEvent e) {
			for (int index = 0; index < gradList.size(); index++) {
				gradRect = gradList.get(index);
				match = gradRect.match(e.getX(), e.getY());
				if (match == true) {
					break;
				}
				gradRect.setSelected(false);
			}
			if (match == true) {
				gradRect.setSelected(true);
			}				
			gradClickPanel.repaint();
		}
	}

	private void gradientKeyCheck() {
		gradScaleT.getInputMap().put(
				KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), "check");
		gradScaleT.getActionMap().put("check", new AbstractAction() {
			private static final long serialVersionUID = -6716014356088669344L;

			public void actionPerformed(ActionEvent e) {
				if (!gradScaleT.isEditValid()) { //The text is invalid.
					Toolkit.getDefaultToolkit().beep();
					gradScaleT.selectAll();
				} else
					try { //The text is valid,
						gradScaleT.commitEdit(); //so use it.
					} catch (java.text.ParseException exc) {
					}
			}
		});
		gradAngleT.getInputMap().put(
				KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), "check");
		gradAngleT.getActionMap().put("check", new AbstractAction() {
			/**
			 * 
			 */
			private static final long serialVersionUID = -3043336258506493172L;

			public void actionPerformed(ActionEvent e) {
				if (!gradAngleT.isEditValid()) { //The text is invalid.
					Toolkit.getDefaultToolkit().beep();
					gradAngleT.selectAll();
				} else
					try { //The text is valid,
						gradAngleT.commitEdit(); //so use it.
					} catch (java.text.ParseException exc) {
					}
			}
		});
		gradOpacityT.getInputMap().put(
				KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), "check");
		gradOpacityT.getActionMap().put("check", new AbstractAction() {
			private static final long serialVersionUID = -3413690850514756341L;

			public void actionPerformed(ActionEvent e) {
				if (!gradOpacityT.isEditValid()) { //The text is invalid.
					Toolkit.getDefaultToolkit().beep();
					gradOpacityT.selectAll();
				} else
					try { //The text is valid,
						gradOpacityT.commitEdit(); //so use it.
					} catch (java.text.ParseException exc) {
					}
			}
		});
	}
}
