package com.sharppaint.printpreview;
import java.awt.*;

import javax.swing.*;

import com.sharppaint.drawtools.Element;

import java.awt.image.VolatileImage;
import java.util.*;

/**print Preview Panel
*/

public class PreviewPanel extends JPanel {
	/**
	 * 
	 */
	private static final long serialVersionUID = 7435926334170554989L;

	private static PreviewPanel previewPanel = null;
	
	private Vector<Element> drawVector;
	
	private Element element;	
	
	private VolatileImage  bimg;
	
	private int width, height;		//for double buffer
	
	private float canvasWidth, canvasHeight;	//canvas size
	
	private static boolean antiMode, renderMode;
	
	private boolean scaleView, scalePrintFlag;
	
	private float scaleCanvas;
	
	private float scalePrint;
	
	private int widthClipSpace;		//clip space
	
	private int heightClipSpace;	//clip space
	
	final static float dash1[] = {3.0f};
	
	final static BasicStroke dashed = new BasicStroke(1.0f, 
            BasicStroke.CAP_BUTT, 
            BasicStroke.JOIN_ROUND, 
            3.0f, dash1, 0.0f);
	
	public static PreviewPanel getInstance() {
		if (previewPanel == null)
			previewPanel = new PreviewPanel();
		return previewPanel;
	}
	private PreviewPanel() {
		setBackground(Color.white);
		drawVector = new Vector<Element>();
		scaleCanvas = 1.0f;
	}
	public void setCanvasSize(float _canvasWidth, float _canvasHeight) {
		canvasWidth=_canvasWidth;
		canvasHeight=_canvasHeight;		
		scaleView = false;
		antiMode=true;
		renderMode=true;
		scalePrint = 1.0f;
	}	
	public void setElement(Element e, int index) {
		drawVector.setElementAt(e, index);
    	repaint();
	}
	public void setAntiMode(boolean _anti) {
		antiMode = _anti;
		repaint();
	}
	public void setRenderMode(boolean _render) {
		renderMode = _render;
		repaint();
	}
	public void setHide(Element e, boolean _hide, int index) {
    	e.setHide(_hide);
    	drawVector.setElementAt(e, index);
    	repaint();
    }
	//drawVector set, getԼ
	public void addVector(Element _elem) {
		drawVector.add(_elem);
	}
	public void setVector(Vector<Element> vector) { 
		drawVector = vector;
	}
	public Vector getVector(){return drawVector;}
	public void initBound(int left, int top, float percent) {
		scaleCanvas = percent;
		if(percent != 1.0)
			scaleView = true;
		else
			scaleView = false;
		this.setBounds(
				left,
				top,
				(int)Math.round(canvasWidth*scaleCanvas),
				(int)Math.round(canvasHeight*scaleCanvas));
		invalidate();
		validate();
		repaint();
	}
	public void setSpace(int _widthClipSpace, int _heightClipSpace) {
		widthClipSpace = _widthClipSpace;
		heightClipSpace = _heightClipSpace;
		invalidate();
		validate();
		repaint();
	}
	public void setScaleSize(float percent) {
		scaleCanvas = percent;
		if(scaleCanvas != 1.0)
			scaleView = true;
		else
			scaleView = false;
		if(scalePrint==1.0f) {
			this.setSize(
					new Dimension(
							(int)Math.round(canvasWidth*scaleCanvas),
							(int)Math.round(canvasHeight*scaleCanvas)));
		} else {
			this.setSize(
					new Dimension(
							(int)Math.round(canvasWidth*scaleCanvas*scalePrint),
							(int)Math.round(canvasHeight*scaleCanvas*scalePrint)));
			
		}
		invalidate();
		validate();
		repaint();
	}
	public void setScalePrint(float _scalePrint) {
		scalePrint = _scalePrint;
		if(scalePrint==1.0f)
			scalePrintFlag = false;
		else
			scalePrintFlag = true;
		this.setSize(
				new Dimension(
						(int)Math.round(canvasWidth*scaleCanvas*scalePrint),
						(int)Math.round(canvasHeight*scaleCanvas*scalePrint)));
		invalidate();
		validate();
		repaint();
	}
	public VolatileImage getBufferImage() {
		return bimg;
	}
	public void paintComponent(Graphics g) {
		super.paintComponent(g);	//panel paintComponent
    	Graphics2D g2 = (Graphics2D)g;
    	Graphics2D g2d = null;
		Dimension d = this.getSize();
    	width = d.width;
    	height = d.height;
    	bimg=null;
	    if (bimg == null || bimg.getWidth() != width
	    		|| bimg.getHeight() != height) {
	       	bimg = (VolatileImage) createVolatileImage(width, height);
	    }
		//shape = new Rectangle(0, 0, Math.round(widthClipSpace*scalePrint), heightClipSpace-1);
	    

	    g2d = bimg.createGraphics();
        g2d.setBackground(getBackground()); 
        g2d.clearRect(0, 0, width, height);
	    g2d.setStroke(dashed);
		//g2d.setStroke(new BasicStroke(1));
		g2d.setColor(Color.BLUE);
		g2d.clipRect(0, 0, widthClipSpace, heightClipSpace);
		g2d.drawRect(0, 0, width-1, height-1);
		if(scalePrintFlag) {
			g2d.scale(scalePrint, scalePrint); 
		}
		//g2d.draw(shape);
    	if(scaleView) {
    		g2d.scale(scaleCanvas, scaleCanvas);	
    	}
        if(antiMode) {
	        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
	               RenderingHints.VALUE_ANTIALIAS_ON);
        }
        if(renderMode) {
			g2d.setRenderingHint(RenderingHints.KEY_RENDERING,
					RenderingHints.VALUE_RENDER_QUALITY);
        }  
		for(int i=0; i<drawVector.size(); i++) {
			element = drawVector.elementAt(i);
			element.draw(g2d);
		}
    	g2.drawImage(bimg, 0, 0, this);
		g2d.dispose();
		g2.dispose();
	}
}
